package Outputsend;

my $pkg="Outputsend";

use strict;
use My;
use Mail::Sendmail;
use MIME::Decoder;
use MIME::QuotedPrint;

#- mailuje dle definice
# priklad:
# Outputsend->mail({
#    From => 'mara@chlup.net',
#    To => 'mara@chlup.net',
#    Subject => 'Test posty',
#    Body => 'Ahoj maro jak se mas?'
# });
# nebo:
# Outputsend->mail({
#    From => 'mara@chlup.net',
#    To => 'mara@chlup.net',
#    Subject => 'Test posty',
#    'Content-type' => 'text/html; charset="iso-8859-2"',
#    Body => '<HTML><HEAD></HEAD><BODY><P>Test zpravy v HTML</P></BODY></HTML>'
# });
sub mail {
   my ($pkg_p, $mail)=@_;
   # zjisteni charset
   my $charset;
   if($$mail{'Content-type'}=~/charset="([^"]+)"/) {
      $charset=$1;
   }
   else {
      $charset="iso-8859-2";
   }
   # prevod quoted-printable
   my $body=encode_qp($$mail{Body});
   my $subject=encode_qp($$mail{Subject}); # druhy argument "" zajisti, ze u dlouhych subjectu nedojde k lamani - neni uplne asi koser, ale v mutt trebas neni problem
   $subject=~s/ /_/g; # mezery nahrazuji podtrzitkem (encode_qp to neudela a mutt pak spatne zobrazuje subject)
   $subject= '=?' . $charset . '?Q?' . $subject . '?=';
   # sestaveni zpravy
   my %smail=(
      From => $$mail{From},
      To => $$mail{To},
      Subject => $subject,
      'Mime-version' => '1.0',
      'Content-Transfer-Encoding' => 'quoted-printable',
      Body => $body
   );
   # Content-type
   if(defined($$mail{'Content-type'})) {
      $smail{'Content-type'}=$$mail{'Content-type'};
   }
   else {
      $smail{'Content-type'}='text/plain; charset="' . $charset . '"';
   }
   # Cc
   if(defined($$mail{Cc})) {
      $smail{Cc}=$$mail{Cc};
   }
   # odeslani zpravy
   return sendmail(%smail);
}

#- tiskne dle definice
# priklad:
# Outputsend->printer({
#    printer => 'lp@opteron.chlup.net',
#    platform => 'printerux',
#    str => "ahoj"
# });
# nebo:
# Outputsend->printer({
#    printer => 'lp@opteron.chlup.net',
#    platform => 'printerux',
#    file => "/tmp/tisk.ps"
# });
sub printer {
   my ($pkg_p, $printer)=@_;
   my ($printername,$host)=split(/\@/,$$printer{printer});
   if(defined($$printer{file})) {
      my $cmd;
      if($$printer{platform} eq "printerux") {
         if($host eq "localhost") {
            $cmd='lpr -P ' . $printername . ' ' . $$printer{file};
         }
         else {
            $cmd='rlpr --printer=' . $$printer{printer} . ' ' . $$printer{file};
         }
      }
      elsif($$printer{platform} eq "printerwin") {
         my ($hostwin)=split(/\./,$host);
         $cmd='smbclient //' . $hostwin . '/' . $printername . ' -I ' . $host . ' -N -P -c "print ' . $$printer{file} . ';"';
      }
      `$cmd`;
   }
   else {
      my $cmd;
      if($$printer{platform} eq "printerux") {
         if($host eq "localhost") {
            $cmd='| lpr -P ' . $printername;
         }
         else {
            $cmd='| rlpr --printer=' . $$printer{printer};
         }
      }
      elsif($$printer{platform} eq "printerwin") {
         my ($hostwin)=split(/\./,$host);
         $cmd='| smbclient //' . $hostwin . '/' . $printername . ' -I ' . $host . ' -N -P -c "print -;"';
      }
      open(CMD,$cmd);
      print CMD $$printer{str};
      close(CMD);
   }
}

#- posila cnc programy na stroj
# priklad:
# Outputsend->cnc({
#    cnc => 'matra',
#    str => "ahoj"
# });
# Poznamka:
# Funkce predpoklada, ze je dobre nastaven seriovy port
# pro dobrou komunikaci s dtc. Nastaveni je mozne
# udelat rucne pomoci programu minicom. Mozna by stacilo
# to automatizovane provadet prikazem stty -F /dev/ttyS0 9600,
# coz vsak nastaveuje jen rychlost a nezabyva se napriklad
# zalezitosti jako 8N1. Ale mozna to bude stacit.
sub cnc {
   my ($pkg_p, $cnc)=@_;
   # jmeno souboru oznacujici zamek a definice na jakem portu bude komunikace probihat
   my $oklock=0;
   my $lock="/tmp/ttyScnc.lock";
   #my $ttyS="/tmp/ttyS0.prac";
   my $ttyS="/dev/ttyS0";
   # vytvoreni zamku pro komunikaci se seriovym portem
   for(my $i=0;$i<5;++$i) {
      if(-e($lock)) {
         sleep 2;
      }
      else {
         open(LOCK,'>'.$lock);
         print LOCK My->date_now;
         print LOCK "\n";
         print LOCK My->remote_user;
         print LOCK "\n";
         print LOCK $$cnc{cnc};
         print LOCK "\n";
         close(LOCK);
         $oklock=1;
         last;
      }
   }
   if($oklock) {
      # prenos souboru
      # Poznamka "\r" == chr(13); "\n" == chr(10)
      open(CMD,'+<'.$ttyS);
#-----------------------------------------------------------------------------
      # napojeni na stroj pres dtc
      print CMD "\r";
      while(defined(my $c=getc(CMD))) {
         print $c;
         if($c eq ">") {
            $c=getc(CMD);
            print $c;
            last;
         }
      }
      print CMD "c ";
      print CMD $$cnc{cnc};
      print CMD "\r";
      while(defined(my $c=getc(CMD))) {
         print $c;
         if($c eq ">") {
            $c=getc(CMD);
            print $c;
            $c=getc(CMD);
            print $c;
            $c=getc(CMD);
            print $c;
            last;
         }
      }
#-----------------------------------------------------------------------------
      # odeslani samtnych dat
      print CMD $$cnc{str};
      if(substr($$cnc{str},-2) ne "\r\n") {
         print CMD "\r\n";
      }
      print CMD chr(17); # znak indikujici zrejme konec prenosu dat
#-----------------------------------------------------------------------------
      # odpojeni od stroje pres dtc
      print CMD chr(11); # Ctrl+k, mozna to je nutne vzdy i bez ohledu na dtc a ze to oznamuje konec prenosu
      while(defined(my $c=getc(CMD))) {
         print $c;
         if($c eq ">") {
            $c=getc(CMD);
            print $c;
            last;
         }
      }
      print CMD "l";
      print CMD "\r";
      while(defined(my $c=getc(CMD))) {
         print $c;
         if($c eq ":") {
            $c=getc(CMD);
            print $c;
            last;
         }
      }
      print CMD "y";
      print CMD "\r";
#-----------------------------------------------------------------------------
      close(CMD);
      # zruseni zamku pro komunikaci se seriovym portem
      unlink($lock);
   }
   return $oklock;
}

return 1;

